# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution	
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools import config
import time

class account_payment_term(osv.osv):
    _name = "account.payment.term"
    _inherit = "account.payment.term"
    _columns = {
        'cash_discount_ids': fields.one2many('account.cash.discount', 'payment_id', 'Cash Discounts'),
    }
account_payment_term()

class account_cash_discount(osv.osv):
    _name = "account.cash.discount"
    _description = "Cash Discount" #A reduction in the price  if payment is made within a stipulated period.
    _columns = {
        'name': fields.char('Name', size=32),
        'delay': fields.integer('Number of Days', required=True),
        'discount': fields.float('Discount (%)', digits=(16,6),required=True),
        'discount_account_id': fields.many2one('account.account', 'Discount Account',required=True),
        'payment_id': fields.many2one('account.payment.term','Associated Payment Term'),
        'credit_account_id': fields.many2one('account.account', 'Credit Account'),
        'debit_account_id': fields.many2one('account.account', 'Debit Account'),
    }
account_cash_discount()

class account_invoice(osv.osv):
    _inherit = "account.invoice"
    
    def _get_payment(self, cr, uid, ids, resudial_amonut, payment_term, context=None):
        """
        This function return the Discount according to the payment term cash discount term
        """
        
        if context is None:
           context = {}
           
        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)
        
        if invoice.date_due:
            date1 = invoice.date_due
        else:
            date1 =  time.strftime('%Y-%m-%d')
                
        if 'date_p' in context and context['date_p']:    
            date2 = context['date_p']
        else:
            date2 =  time.strftime('%Y-%m-%d')   

        from_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
        to_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
        diff_day = (from_dt-to_dt)/(3600*24)
        if payment_term:
            pt = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if pt.cash_discount_ids:
                    res = 0.0
                    for d in pt.cash_discount_ids:
                        if diff_day == d.delay or diff_day < d.delay:
                            if resudial_amonut:
                                res = resudial_amonut * d.discount
                            ted = 0.0
                            for line in invoice.invoice_line:
                                if line.invoice_line_tax_id:
                                    tt = tax_obj.compute_inv(cr, uid, line.invoice_line_tax_id, res, line.quantity)
                                    for tex in tt:
                                        rr = 0.0 
                                        rr += tex['amount']
                                    ted +=rr
                            discunt = res + ted
                            return discunt
     
    def _get_account(self, cr, uid, ids, resudial_amonut, payment_term, context=None):
        """
        This function return the Account according to the payment term cash discount term
        """
        if context is None:
           context = {}

        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)
        
        if invoice.date_due:
            date1 = invoice.date_due
        else:
            date1 =  time.strftime('%Y-%m-%d')
                
        if 'date_p' in context and context['date_p']:    
            date2 = context['date_p']
        else:
            date2 =  time.strftime('%Y-%m-%d')   

        from_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
        to_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
        diff_day = (from_dt-to_dt)/(3600*24)
        if payment_term:
            pt = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if pt.cash_discount_ids:
                    res = 0.0
                    for d in pt.cash_discount_ids:
                        if diff_day == d.delay or diff_day < d.delay:
                            account_id = d.discount_account_id.id
                            return account_id 
            
    def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context=None, name=''):
        
        if context is None:
            context = {}
        #TODO check if we can use different period for payment and the writeoff line
        assert len(ids)==1, "Can only pay one invoice at a time"
        invoice = self.browse(cr, uid, ids[0])
        
        if 'amount_currency' in context and context['amount_currency']:
            amount_currency = context['amount_currency']
        else:
            amount_currency = invoice.residual

        if invoice.payment_term:
            # Return the discount on for the payment term
            discunt1 = self._get_payment(cr, uid, ids, amount_currency, invoice.payment_term.id, context=context)

        src_account_id = invoice.account_id.id
        # Take the seq as name for move
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        #take the choosen date
        if 'date_p' in context and context['date_p']:
            date=context['date_p']
        else:
            date=time.strftime('%Y-%m-%d')
            
        # Take the amount in currency and the currency of the payment
        if 'amount_currency' in context and context['amount_currency'] and 'currency_id' in context and context['currency_id']:
            amount_currency = context['amount_currency']
            currency_id = context['currency_id']
        else:
            amount_currency = False
            currency_id = False
        
        if invoice.type in ('in_invoice', 'in_refund'):
            ref = invoice.reference
        else:
            ref = self._convert_ref(cr, uid, invoice.number)        
        # Pay attention to the sign for both debit/credit AND amount_currency
        l1 = {
            'debit': direction * (pay_amount)>0 and direction * (pay_amount),
            'credit': direction * (pay_amount)<0 and - direction * (pay_amount),
            'account_id': src_account_id,
            'partner_id': invoice.partner_id.id,
            'ref':ref,
            'date': date,
            'currency_id':currency_id,
            'amount_currency':amount_currency and direction * amount_currency or 0.0,
        }
        l2 = {
            'debit': direction * (pay_amount-discunt1)<0 and - direction * (pay_amount-discunt1),
            'credit': direction * (pay_amount-discunt1)>0 and direction * (pay_amount-discunt1),
            'account_id': pay_account_id,
            'partner_id': invoice.partner_id.id,
            'ref':ref,
            'date': date,
            'currency_id':currency_id,
            'amount_currency':amount_currency and - direction * amount_currency or 0.0,
        }
        l3 = {
            'debit': direction * discunt1<0 and - direction * discunt1,
            'credit': direction * discunt1>0 and  direction * discunt1,
            'account_id': context['account_id'],
            'partner_id': invoice.partner_id.id,
            'ref':ref,
            'date': date,
            'currency_id':currency_id,
            'amount_currency':amount_currency and direction * amount_currency or 0.0,
        }

        if not name:
            name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number
        l1['name'] = name
        l2['name'] = name
        l3['name'] = name
        lines = [(0, 0, l1), (0, 0, l2), (0, 0, l3)]

        move = {'ref': ref, 'line_id': lines, 'journal_id': pay_journal_id, 'period_id': period_id, 'date': date}
        move_id = self.pool.get('account.move').create(cr, uid, move, context=context)

        line_ids = []
        total = 0.0
        line = self.pool.get('account.move.line')
        cr.execute('select id from account_move_line where move_id in ('+str(move_id)+','+str(invoice.move_id.id)+')')
        lines = line.browse(cr, uid, map(lambda x: x[0], cr.fetchall()) )
        for l in lines+invoice.payment_ids:
            if l.account_id.id==src_account_id:
                line_ids.append(l.id)
                total += (l.debit or 0.0) - (l.credit or 0.0)
        if (not round(total,int(config['price_accuracy']))) or writeoff_acc_id:
            self.pool.get('account.move.line').reconcile(cr, uid, line_ids, 'manual', writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
        else:
            self.pool.get('account.move.line').reconcile_partial(cr, uid, line_ids, 'manual', context)

        # Update the stored value (fields.function), so we write to trigger recompute
        self.pool.get('account.invoice').write(cr, uid, ids, {}, context=context)
        return True
    
account_invoice()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

